#*********************************************************************
#*
#* $Id: pic24config.php 26169 2016-12-12 01:36:34Z mvuilleu $
#*
#* Implements yFindProximity(), the high-level API for Proximity functions
#*
#* - - - - - - - - - License information: - - - - - - - - - 
#*
#*  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#*
#*  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#*  non-exclusive license to use, modify, copy and integrate this
#*  file into your software for the sole purpose of interfacing
#*  with Yoctopuce products.
#*
#*  You may reproduce and distribute copies of this file in
#*  source or object form, as long as the sole purpose of this
#*  code is to interface with Yoctopuce products. You must retain
#*  this notice in the distributed source file.
#*
#*  You should refer to Yoctopuce General Terms and Conditions
#*  for additional information regarding your rights and
#*  obligations.
#*
#*  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#*  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#*  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#*  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#*  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#*  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#*  WARRANTY, OR OTHERWISE.
#*
#*********************************************************************/


__docformat__ = 'restructuredtext en'
from yoctopuce.yocto_api import *


#--- (YProximity class start)
#noinspection PyProtectedMember
class YProximity(YSensor):
    """
    The Yoctopuce class YProximity allows you to use and configure Yoctopuce proximity
    sensors. It inherits from YSensor class the core functions to read measurements,
    register callback functions, access to the autonomous datalogger.
    This class adds the ability to easily perform a one-point linear calibration
    to compensate the effect of a glass or filter placed in front of the sensor.

    """
#--- (end of YProximity class start)
    #--- (YProximity return codes)
    #--- (end of YProximity return codes)
    #--- (YProximity dlldef)
    #--- (end of YProximity dlldef)
    #--- (YProximity definitions)
    DETECTIONTHRESHOLD_INVALID = YAPI.INVALID_UINT
    LASTTIMEAPPROACHED_INVALID = YAPI.INVALID_LONG
    LASTTIMEREMOVED_INVALID = YAPI.INVALID_LONG
    PULSECOUNTER_INVALID = YAPI.INVALID_LONG
    PULSETIMER_INVALID = YAPI.INVALID_LONG
    ISPRESENT_FALSE = 0
    ISPRESENT_TRUE = 1
    ISPRESENT_INVALID = -1
    #--- (end of YProximity definitions)

    def __init__(self, func):
        super(YProximity, self).__init__(func)
        self._className = 'Proximity'
        #--- (YProximity attributes)
        self._callback = None
        self._detectionThreshold = YProximity.DETECTIONTHRESHOLD_INVALID
        self._isPresent = YProximity.ISPRESENT_INVALID
        self._lastTimeApproached = YProximity.LASTTIMEAPPROACHED_INVALID
        self._lastTimeRemoved = YProximity.LASTTIMEREMOVED_INVALID
        self._pulseCounter = YProximity.PULSECOUNTER_INVALID
        self._pulseTimer = YProximity.PULSETIMER_INVALID
        #--- (end of YProximity attributes)

    #--- (YProximity implementation)
    def _parseAttr(self, member):
        if member.name == "detectionThreshold":
            self._detectionThreshold = member.ivalue
            return 1
        if member.name == "isPresent":
            self._isPresent = member.ivalue
            return 1
        if member.name == "lastTimeApproached":
            self._lastTimeApproached = member.ivalue
            return 1
        if member.name == "lastTimeRemoved":
            self._lastTimeRemoved = member.ivalue
            return 1
        if member.name == "pulseCounter":
            self._pulseCounter = member.ivalue
            return 1
        if member.name == "pulseTimer":
            self._pulseTimer = member.ivalue
            return 1
        super(YProximity, self)._parseAttr(member)

    def get_detectionThreshold(self):
        """
        Returns the threshold used to determine the logical state of the proximity sensor, when considered
        as a binary input (on/off).

        @return an integer corresponding to the threshold used to determine the logical state of the
        proximity sensor, when considered
                as a binary input (on/off)

        On failure, throws an exception or returns YProximity.DETECTIONTHRESHOLD_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YProximity.DETECTIONTHRESHOLD_INVALID
        return self._detectionThreshold

    def set_detectionThreshold(self, newval):
        """
        Changes the threshold used to determine the logical state of the proximity sensor, when considered
        as a binary input (on/off).

        @param newval : an integer corresponding to the threshold used to determine the logical state of
        the proximity sensor, when considered
                as a binary input (on/off)

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return self._setAttr("detectionThreshold", rest_val)

    def get_isPresent(self):
        """
        Returns true if the input (considered as binary) is active (detection value is smaller than the
        specified threshold), and false otherwise.

        @return either YProximity.ISPRESENT_FALSE or YProximity.ISPRESENT_TRUE, according to true if the
        input (considered as binary) is active (detection value is smaller than the specified threshold),
        and false otherwise

        On failure, throws an exception or returns YProximity.ISPRESENT_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YProximity.ISPRESENT_INVALID
        return self._isPresent

    def get_lastTimeApproached(self):
        """
        Returns the number of elapsed milliseconds between the module power on and the last observed
        detection (the input contact transitioned from absent to present).

        @return an integer corresponding to the number of elapsed milliseconds between the module power on
        and the last observed
                detection (the input contact transitioned from absent to present)

        On failure, throws an exception or returns YProximity.LASTTIMEAPPROACHED_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YProximity.LASTTIMEAPPROACHED_INVALID
        return self._lastTimeApproached

    def get_lastTimeRemoved(self):
        """
        Returns the number of elapsed milliseconds between the module power on and the last observed
        detection (the input contact transitioned from present to absent).

        @return an integer corresponding to the number of elapsed milliseconds between the module power on
        and the last observed
                detection (the input contact transitioned from present to absent)

        On failure, throws an exception or returns YProximity.LASTTIMEREMOVED_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YProximity.LASTTIMEREMOVED_INVALID
        return self._lastTimeRemoved

    def get_pulseCounter(self):
        """
        Returns the pulse counter value. The value is a 32 bit integer. In case
        of overflow (>=2^32), the counter will wrap. To reset the counter, just
        call the resetCounter() method.

        @return an integer corresponding to the pulse counter value

        On failure, throws an exception or returns YProximity.PULSECOUNTER_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YProximity.PULSECOUNTER_INVALID
        return self._pulseCounter

    def set_pulseCounter(self, newval):
        rest_val = str(newval)
        return self._setAttr("pulseCounter", rest_val)

    def get_pulseTimer(self):
        """
        Returns the timer of the pulses counter (ms).

        @return an integer corresponding to the timer of the pulses counter (ms)

        On failure, throws an exception or returns YProximity.PULSETIMER_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YProximity.PULSETIMER_INVALID
        return self._pulseTimer

    @staticmethod
    def FindProximity(func):
        """
        Retrieves a proximity sensor for a given identifier.
        The identifier can be specified using several formats:
        <ul>
        <li>FunctionLogicalName</li>
        <li>ModuleSerialNumber.FunctionIdentifier</li>
        <li>ModuleSerialNumber.FunctionLogicalName</li>
        <li>ModuleLogicalName.FunctionIdentifier</li>
        <li>ModuleLogicalName.FunctionLogicalName</li>
        </ul>

        This function does not require that the proximity sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YProximity.isOnline() to test if the proximity sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a proximity sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param func : a string that uniquely characterizes the proximity sensor

        @return a YProximity object allowing you to drive the proximity sensor.
        """
        # obj
        obj = YFunction._FindFromCache("Proximity", func)
        if obj is None:
            obj = YProximity(func)
            YFunction._AddToCache("Proximity", func, obj)
        return obj

    def resetCounter(self):
        """
        Returns the pulse counter value as well as its timer.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return self.set_pulseCounter(0)

    def nextProximity(self):
        """
        Continues the enumeration of proximity sensors started using yFirstProximity().

        @return a pointer to a YProximity object, corresponding to
                a proximity sensor currently online, or a None pointer
                if there are no more proximity sensors to enumerate.
        """
        hwidRef = YRefParam()
        if YAPI.YISERR(self._nextFunction(hwidRef)):
            return None
        if hwidRef.value == "":
            return None
        return YProximity.FindProximity(hwidRef.value)

#--- (end of YProximity implementation)

#--- (Proximity functions)

    @staticmethod
    def FirstProximity():
        """
        Starts the enumeration of proximity sensors currently accessible.
        Use the method YProximity.nextProximity() to iterate on
        next proximity sensors.

        @return a pointer to a YProximity object, corresponding to
                the first proximity sensor currently online, or a None pointer
                if there are none.
        """
        devRef = YRefParam()
        neededsizeRef = YRefParam()
        serialRef = YRefParam()
        funcIdRef = YRefParam()
        funcNameRef = YRefParam()
        funcValRef = YRefParam()
        errmsgRef = YRefParam()
        size = YAPI.C_INTSIZE
        #noinspection PyTypeChecker,PyCallingNonCallable
        p = (ctypes.c_int * 1)()
        err = YAPI.apiGetFunctionsByClass("Proximity", 0, p, size, neededsizeRef, errmsgRef)

        if YAPI.YISERR(err) or not neededsizeRef.value:
            return None

        if YAPI.YISERR(
                YAPI.yapiGetFunctionInfo(p[0], devRef, serialRef, funcIdRef, funcNameRef, funcValRef, errmsgRef)):
            return None

        return YProximity.FindProximity(serialRef.value + "." + funcIdRef.value)

#--- (end of Proximity functions)



#*********************************************************************
#*
#* $Id: yocto_pressure.py 23243 2016-02-23 14:13:12Z seb $
#*
#* Implements yFindPressure(), the high-level API for Pressure functions
#*
#* - - - - - - - - - License information: - - - - - - - - - 
#*
#*  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#*
#*  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#*  non-exclusive license to use, modify, copy and integrate this
#*  file into your software for the sole purpose of interfacing
#*  with Yoctopuce products.
#*
#*  You may reproduce and distribute copies of this file in
#*  source or object form, as long as the sole purpose of this
#*  code is to interface with Yoctopuce products. You must retain
#*  this notice in the distributed source file.
#*
#*  You should refer to Yoctopuce General Terms and Conditions
#*  for additional information regarding your rights and
#*  obligations.
#*
#*  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#*  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#*  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#*  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#*  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#*  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#*  WARRANTY, OR OTHERWISE.
#*
#*********************************************************************/


__docformat__ = 'restructuredtext en'
from yoctopuce.yocto_api import *


#--- (YPressure class start)
#noinspection PyProtectedMember
class YPressure(YSensor):
    """
    The Yoctopuce class YPressure allows you to read and configure Yoctopuce pressure
    sensors. It inherits from YSensor class the core functions to read measurements,
    register callback functions, access to the autonomous datalogger.

    """
#--- (end of YPressure class start)
    #--- (YPressure return codes)
    #--- (end of YPressure return codes)
    #--- (YPressure dlldef)
    #--- (end of YPressure dlldef)
    #--- (YPressure definitions)
    #--- (end of YPressure definitions)

    def __init__(self, func):
        super(YPressure, self).__init__(func)
        self._className = 'Pressure'
        #--- (YPressure attributes)
        self._callback = None
        #--- (end of YPressure attributes)

    #--- (YPressure implementation)
    def _parseAttr(self, member):
        super(YPressure, self)._parseAttr(member)

    @staticmethod
    def FindPressure(func):
        """
        Retrieves a pressure sensor for a given identifier.
        The identifier can be specified using several formats:
        <ul>
        <li>FunctionLogicalName</li>
        <li>ModuleSerialNumber.FunctionIdentifier</li>
        <li>ModuleSerialNumber.FunctionLogicalName</li>
        <li>ModuleLogicalName.FunctionIdentifier</li>
        <li>ModuleLogicalName.FunctionLogicalName</li>
        </ul>

        This function does not require that the pressure sensor is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YPressure.isOnline() to test if the pressure sensor is
        indeed online at a given time. In case of ambiguity when looking for
        a pressure sensor by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param func : a string that uniquely characterizes the pressure sensor

        @return a YPressure object allowing you to drive the pressure sensor.
        """
        # obj
        obj = YFunction._FindFromCache("Pressure", func)
        if obj is None:
            obj = YPressure(func)
            YFunction._AddToCache("Pressure", func, obj)
        return obj

    def nextPressure(self):
        """
        Continues the enumeration of pressure sensors started using yFirstPressure().

        @return a pointer to a YPressure object, corresponding to
                a pressure sensor currently online, or a None pointer
                if there are no more pressure sensors to enumerate.
        """
        hwidRef = YRefParam()
        if YAPI.YISERR(self._nextFunction(hwidRef)):
            return None
        if hwidRef.value == "":
            return None
        return YPressure.FindPressure(hwidRef.value)

#--- (end of YPressure implementation)

#--- (Pressure functions)

    @staticmethod
    def FirstPressure():
        """
        Starts the enumeration of pressure sensors currently accessible.
        Use the method YPressure.nextPressure() to iterate on
        next pressure sensors.

        @return a pointer to a YPressure object, corresponding to
                the first pressure sensor currently online, or a None pointer
                if there are none.
        """
        devRef = YRefParam()
        neededsizeRef = YRefParam()
        serialRef = YRefParam()
        funcIdRef = YRefParam()
        funcNameRef = YRefParam()
        funcValRef = YRefParam()
        errmsgRef = YRefParam()
        size = YAPI.C_INTSIZE
        #noinspection PyTypeChecker,PyCallingNonCallable
        p = (ctypes.c_int * 1)()
        err = YAPI.apiGetFunctionsByClass("Pressure", 0, p, size, neededsizeRef, errmsgRef)

        if YAPI.YISERR(err) or not neededsizeRef.value:
            return None

        if YAPI.YISERR(
                YAPI.yapiGetFunctionInfo(p[0], devRef, serialRef, funcIdRef, funcNameRef, funcValRef, errmsgRef)):
            return None

        return YPressure.FindPressure(serialRef.value + "." + funcIdRef.value)

#--- (end of Pressure functions)



#*********************************************************************
#*
#* $Id: yocto_digitalio.py 25871 2016-11-15 14:32:56Z seb $
#*
#* Implements yFindDigitalIO(), the high-level API for DigitalIO functions
#*
#* - - - - - - - - - License information: - - - - - - - - - 
#*
#*  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#*
#*  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#*  non-exclusive license to use, modify, copy and integrate this
#*  file into your software for the sole purpose of interfacing
#*  with Yoctopuce products.
#*
#*  You may reproduce and distribute copies of this file in
#*  source or object form, as long as the sole purpose of this
#*  code is to interface with Yoctopuce products. You must retain
#*  this notice in the distributed source file.
#*
#*  You should refer to Yoctopuce General Terms and Conditions
#*  for additional information regarding your rights and
#*  obligations.
#*
#*  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING 
#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#*  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#*  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#*  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#*  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#*  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#*  WARRANTY, OR OTHERWISE.
#*
#*********************************************************************/


__docformat__ = 'restructuredtext en'
from yoctopuce.yocto_api import *


#--- (YDigitalIO class start)
#noinspection PyProtectedMember
class YDigitalIO(YFunction):
    """
    The Yoctopuce application programming interface allows you to switch the state of each
    bit of the I/O port. You can switch all bits at once, or one by one. The library
    can also automatically generate short pulses of a determined duration. Electrical behavior
    of each I/O can be modified (open drain and reverse polarity).

    """
#--- (end of YDigitalIO class start)
    #--- (YDigitalIO return codes)
    #--- (end of YDigitalIO return codes)
    #--- (YDigitalIO dlldef)
    #--- (end of YDigitalIO dlldef)
    #--- (YDigitalIO definitions)
    PORTSTATE_INVALID = YAPI.INVALID_UINT
    PORTDIRECTION_INVALID = YAPI.INVALID_UINT
    PORTOPENDRAIN_INVALID = YAPI.INVALID_UINT
    PORTPOLARITY_INVALID = YAPI.INVALID_UINT
    PORTSIZE_INVALID = YAPI.INVALID_UINT
    COMMAND_INVALID = YAPI.INVALID_STRING
    OUTPUTVOLTAGE_USB_5V = 0
    OUTPUTVOLTAGE_USB_3V = 1
    OUTPUTVOLTAGE_EXT_V = 2
    OUTPUTVOLTAGE_INVALID = -1
    #--- (end of YDigitalIO definitions)

    def __init__(self, func):
        super(YDigitalIO, self).__init__(func)
        self._className = 'DigitalIO'
        #--- (YDigitalIO attributes)
        self._callback = None
        self._portState = YDigitalIO.PORTSTATE_INVALID
        self._portDirection = YDigitalIO.PORTDIRECTION_INVALID
        self._portOpenDrain = YDigitalIO.PORTOPENDRAIN_INVALID
        self._portPolarity = YDigitalIO.PORTPOLARITY_INVALID
        self._portSize = YDigitalIO.PORTSIZE_INVALID
        self._outputVoltage = YDigitalIO.OUTPUTVOLTAGE_INVALID
        self._command = YDigitalIO.COMMAND_INVALID
        #--- (end of YDigitalIO attributes)

    #--- (YDigitalIO implementation)
    def _parseAttr(self, member):
        if member.name == "portState":
            self._portState = member.ivalue
            return 1
        if member.name == "portDirection":
            self._portDirection = member.ivalue
            return 1
        if member.name == "portOpenDrain":
            self._portOpenDrain = member.ivalue
            return 1
        if member.name == "portPolarity":
            self._portPolarity = member.ivalue
            return 1
        if member.name == "portSize":
            self._portSize = member.ivalue
            return 1
        if member.name == "outputVoltage":
            self._outputVoltage = member.ivalue
            return 1
        if member.name == "command":
            self._command = member.svalue
            return 1
        super(YDigitalIO, self)._parseAttr(member)

    def get_portState(self):
        """
        Returns the digital IO port state: bit 0 represents input 0, and so on.

        @return an integer corresponding to the digital IO port state: bit 0 represents input 0, and so on

        On failure, throws an exception or returns YDigitalIO.PORTSTATE_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YDigitalIO.PORTSTATE_INVALID
        return self._portState

    def set_portState(self, newval):
        """
        Changes the digital IO port state: bit 0 represents input 0, and so on. This function has no effect
        on bits configured as input in portDirection.

        @param newval : an integer corresponding to the digital IO port state: bit 0 represents input 0, and so on

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return self._setAttr("portState", rest_val)

    def get_portDirection(self):
        """
        Returns the IO direction of all bits of the port: 0 makes a bit an input, 1 makes it an output.

        @return an integer corresponding to the IO direction of all bits of the port: 0 makes a bit an
        input, 1 makes it an output

        On failure, throws an exception or returns YDigitalIO.PORTDIRECTION_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YDigitalIO.PORTDIRECTION_INVALID
        return self._portDirection

    def set_portDirection(self, newval):
        """
        Changes the IO direction of all bits of the port: 0 makes a bit an input, 1 makes it an output.
        Remember to call the saveToFlash() method  to make sure the setting is kept after a reboot.

        @param newval : an integer corresponding to the IO direction of all bits of the port: 0 makes a bit
        an input, 1 makes it an output

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return self._setAttr("portDirection", rest_val)

    def get_portOpenDrain(self):
        """
        Returns the electrical interface for each bit of the port. For each bit set to 0  the matching I/O
        works in the regular,
        intuitive way, for each bit set to 1, the I/O works in reverse mode.

        @return an integer corresponding to the electrical interface for each bit of the port

        On failure, throws an exception or returns YDigitalIO.PORTOPENDRAIN_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YDigitalIO.PORTOPENDRAIN_INVALID
        return self._portOpenDrain

    def set_portOpenDrain(self, newval):
        """
        Changes the electrical interface for each bit of the port. 0 makes a bit a regular input/output, 1 makes
        it an open-drain (open-collector) input/output. Remember to call the
        saveToFlash() method  to make sure the setting is kept after a reboot.

        @param newval : an integer corresponding to the electrical interface for each bit of the port

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return self._setAttr("portOpenDrain", rest_val)

    def get_portPolarity(self):
        """
        Returns the polarity of all the bits of the port.  For each bit set to 0, the matching I/O works the regular,
        intuitive way; for each bit set to 1, the I/O works in reverse mode.

        @return an integer corresponding to the polarity of all the bits of the port

        On failure, throws an exception or returns YDigitalIO.PORTPOLARITY_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YDigitalIO.PORTPOLARITY_INVALID
        return self._portPolarity

    def set_portPolarity(self, newval):
        """
        Changes the polarity of all the bits of the port: For each bit set to 0, the matching I/O works the regular,
        intuitive way; for each bit set to 1, the I/O works in reverse mode.
        Remember to call the saveToFlash() method  to make sure the setting will be kept after a reboot.

        @param newval : an integer corresponding to the polarity of all the bits of the port: For each bit
        set to 0, the matching I/O works the regular,
                intuitive way; for each bit set to 1, the I/O works in reverse mode

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return self._setAttr("portPolarity", rest_val)

    def get_portSize(self):
        """
        Returns the number of bits implemented in the I/O port.

        @return an integer corresponding to the number of bits implemented in the I/O port

        On failure, throws an exception or returns YDigitalIO.PORTSIZE_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YDigitalIO.PORTSIZE_INVALID
        return self._portSize

    def get_outputVoltage(self):
        """
        Returns the voltage source used to drive output bits.

        @return a value among YDigitalIO.OUTPUTVOLTAGE_USB_5V, YDigitalIO.OUTPUTVOLTAGE_USB_3V and
        YDigitalIO.OUTPUTVOLTAGE_EXT_V corresponding to the voltage source used to drive output bits

        On failure, throws an exception or returns YDigitalIO.OUTPUTVOLTAGE_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YDigitalIO.OUTPUTVOLTAGE_INVALID
        return self._outputVoltage

    def set_outputVoltage(self, newval):
        """
        Changes the voltage source used to drive output bits.
        Remember to call the saveToFlash() method  to make sure the setting is kept after a reboot.

        @param newval : a value among YDigitalIO.OUTPUTVOLTAGE_USB_5V, YDigitalIO.OUTPUTVOLTAGE_USB_3V and
        YDigitalIO.OUTPUTVOLTAGE_EXT_V corresponding to the voltage source used to drive output bits

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return self._setAttr("outputVoltage", rest_val)

    def get_command(self):
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YDigitalIO.COMMAND_INVALID
        return self._command

    def set_command(self, newval):
        rest_val = newval
        return self._setAttr("command", rest_val)

    @staticmethod
    def FindDigitalIO(func):
        """
        Retrieves a digital IO port for a given identifier.
        The identifier can be specified using several formats:
        <ul>
        <li>FunctionLogicalName</li>
        <li>ModuleSerialNumber.FunctionIdentifier</li>
        <li>ModuleSerialNumber.FunctionLogicalName</li>
        <li>ModuleLogicalName.FunctionIdentifier</li>
        <li>ModuleLogicalName.FunctionLogicalName</li>
        </ul>

        This function does not require that the digital IO port is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YDigitalIO.isOnline() to test if the digital IO port is
        indeed online at a given time. In case of ambiguity when looking for
        a digital IO port by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param func : a string that uniquely characterizes the digital IO port

        @return a YDigitalIO object allowing you to drive the digital IO port.
        """
        # obj
        obj = YFunction._FindFromCache("DigitalIO", func)
        if obj is None:
            obj = YDigitalIO(func)
            YFunction._AddToCache("DigitalIO", func, obj)
        return obj

    def set_bitState(self, bitno, bitstate):
        """
        Sets a single bit of the I/O port.

        @param bitno : the bit number; lowest bit has index 0
        @param bitstate : the state of the bit (1 or 0)

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        if not (bitstate >= 0):
            self._throw(YAPI.INVALID_ARGUMENT, "invalid bitstate")
        if not (bitstate <= 1):
            self._throw(YAPI.INVALID_ARGUMENT, "invalid bitstate")
        return self.set_command("" + str(chr(82+bitstate)) + "" + str(int(bitno)))

    def get_bitState(self, bitno):
        """
        Returns the state of a single bit of the I/O port.

        @param bitno : the bit number; lowest bit has index 0

        @return the bit state (0 or 1)

        On failure, throws an exception or returns a negative error code.
        """
        # portVal
        portVal = self.get_portState()
        return ((((portVal) >> (bitno))) & (1))

    def toggle_bitState(self, bitno):
        """
        Reverts a single bit of the I/O port.

        @param bitno : the bit number; lowest bit has index 0

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return self.set_command("T" + str(int(bitno)))

    def set_bitDirection(self, bitno, bitdirection):
        """
        Changes  the direction of a single bit from the I/O port.

        @param bitno : the bit number; lowest bit has index 0
        @param bitdirection : direction to set, 0 makes the bit an input, 1 makes it an output.
                Remember to call the   saveToFlash() method to make sure the setting is kept after a reboot.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        if not (bitdirection >= 0):
            self._throw(YAPI.INVALID_ARGUMENT, "invalid direction")
        if not (bitdirection <= 1):
            self._throw(YAPI.INVALID_ARGUMENT, "invalid direction")
        return self.set_command("" + str(chr(73+6*bitdirection)) + "" + str(int(bitno)))

    def get_bitDirection(self, bitno):
        """
        Returns the direction of a single bit from the I/O port (0 means the bit is an input, 1  an output).

        @param bitno : the bit number; lowest bit has index 0

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        # portDir
        portDir = self.get_portDirection()
        return ((((portDir) >> (bitno))) & (1))

    def set_bitPolarity(self, bitno, bitpolarity):
        """
        Changes the polarity of a single bit from the I/O port.

        @param bitno : the bit number; lowest bit has index 0.
        @param bitpolarity : polarity to set, 0 makes the I/O work in regular mode, 1 makes the I/O  works
        in reverse mode.
                Remember to call the   saveToFlash() method to make sure the setting is kept after a reboot.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        if not (bitpolarity >= 0):
            self._throw(YAPI.INVALID_ARGUMENT, "invalid bitpolarity")
        if not (bitpolarity <= 1):
            self._throw(YAPI.INVALID_ARGUMENT, "invalid bitpolarity")
        return self.set_command("" + str(chr(110+4*bitpolarity)) + "" + str(int(bitno)))

    def get_bitPolarity(self, bitno):
        """
        Returns the polarity of a single bit from the I/O port (0 means the I/O works in regular mode, 1
        means the I/O  works in reverse mode).

        @param bitno : the bit number; lowest bit has index 0

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        # portPol
        portPol = self.get_portPolarity()
        return ((((portPol) >> (bitno))) & (1))

    def set_bitOpenDrain(self, bitno, opendrain):
        """
        Changes  the electrical interface of a single bit from the I/O port.

        @param bitno : the bit number; lowest bit has index 0
        @param opendrain : 0 makes a bit a regular input/output, 1 makes
                it an open-drain (open-collector) input/output. Remember to call the
                saveToFlash() method to make sure the setting is kept after a reboot.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        if not (opendrain >= 0):
            self._throw(YAPI.INVALID_ARGUMENT, "invalid state")
        if not (opendrain <= 1):
            self._throw(YAPI.INVALID_ARGUMENT, "invalid state")
        return self.set_command("" + str(chr(100-32*opendrain)) + "" + str(int(bitno)))

    def get_bitOpenDrain(self, bitno):
        """
        Returns the type of electrical interface of a single bit from the I/O port. (0 means the bit is an
        input, 1  an output).

        @param bitno : the bit number; lowest bit has index 0

        @return   0 means the a bit is a regular input/output, 1 means the bit is an open-drain
                (open-collector) input/output.

        On failure, throws an exception or returns a negative error code.
        """
        # portOpenDrain
        portOpenDrain = self.get_portOpenDrain()
        return ((((portOpenDrain) >> (bitno))) & (1))

    def pulse(self, bitno, ms_duration):
        """
        Triggers a pulse on a single bit for a specified duration. The specified bit
        will be turned to 1, and then back to 0 after the given duration.

        @param bitno : the bit number; lowest bit has index 0
        @param ms_duration : desired pulse duration in milliseconds. Be aware that the device time
                resolution is not guaranteed up to the millisecond.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return self.set_command("Z" + str(int(bitno)) + ",0," + str(int(ms_duration)))

    def delayedPulse(self, bitno, ms_delay, ms_duration):
        """
        Schedules a pulse on a single bit for a specified duration. The specified bit
        will be turned to 1, and then back to 0 after the given duration.

        @param bitno : the bit number; lowest bit has index 0
        @param ms_delay : waiting time before the pulse, in milliseconds
        @param ms_duration : desired pulse duration in milliseconds. Be aware that the device time
                resolution is not guaranteed up to the millisecond.

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        return self.set_command("Z" + str(int(bitno)) + "," + str(int(ms_delay)) + "," + str(int(ms_duration)))

    def nextDigitalIO(self):
        """
        Continues the enumeration of digital IO ports started using yFirstDigitalIO().

        @return a pointer to a YDigitalIO object, corresponding to
                a digital IO port currently online, or a None pointer
                if there are no more digital IO ports to enumerate.
        """
        hwidRef = YRefParam()
        if YAPI.YISERR(self._nextFunction(hwidRef)):
            return None
        if hwidRef.value == "":
            return None
        return YDigitalIO.FindDigitalIO(hwidRef.value)

#--- (end of YDigitalIO implementation)

#--- (DigitalIO functions)

    @staticmethod
    def FirstDigitalIO():
        """
        Starts the enumeration of digital IO ports currently accessible.
        Use the method YDigitalIO.nextDigitalIO() to iterate on
        next digital IO ports.

        @return a pointer to a YDigitalIO object, corresponding to
                the first digital IO port currently online, or a None pointer
                if there are none.
        """
        devRef = YRefParam()
        neededsizeRef = YRefParam()
        serialRef = YRefParam()
        funcIdRef = YRefParam()
        funcNameRef = YRefParam()
        funcValRef = YRefParam()
        errmsgRef = YRefParam()
        size = YAPI.C_INTSIZE
        #noinspection PyTypeChecker,PyCallingNonCallable
        p = (ctypes.c_int * 1)()
        err = YAPI.apiGetFunctionsByClass("DigitalIO", 0, p, size, neededsizeRef, errmsgRef)

        if YAPI.YISERR(err) or not neededsizeRef.value:
            return None

        if YAPI.YISERR(
                YAPI.yapiGetFunctionInfo(p[0], devRef, serialRef, funcIdRef, funcNameRef, funcValRef, errmsgRef)):
            return None

        return YDigitalIO.FindDigitalIO(serialRef.value + "." + funcIdRef.value)

#--- (end of DigitalIO functions)



#!/usr/bin/python
# -*- coding: utf-8 -*-
import os, sys

# add ../../Sources to the PYTHONPATH
sys.path.append(os.path.join("..", "..", "Sources"))
from yoctopuce.yocto_api import *
from yoctopuce.yocto_spiport import *


def usage():
    scriptname = os.path.basename(sys.argv[0])
    print("Usage:")
    print(scriptname + " <serial_number>  <value>")
    print(scriptname + " <logical_name>   <value>")
    print(scriptname + " any  <value>   (use any discovered device)")
    sys.exit()


def die(msg):
    sys.exit(msg + ' (check USB cable)')


if len(sys.argv) < 3:
    usage()
target = sys.argv[1].upper()
value = int(sys.argv[2])

# Setup the API to use local USB devices. You can
# use an IP address instead of 'usb' if the device
# is connected to a network.
errmsg = YRefParam()
if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
    sys.exit("init error" + errmsg.value)

if target == 'ANY':
    spiPort = YSpiPort.FirstSpiPort()
    if spiPort is None:
        sys.exit('No module connected (check cable)')
else:
    spiPort = YSpiPort.FindSpiPort(sys.argv[1] + ".spiPort")
    if not spiPort.isOnline():
        sys.exit('Module not connected')

# sample code driving MAX7219 7-segment display driver
# such as SPI7SEGDISP8.56 from www.embedded-lab.com
spiPort.set_spiMode("250000,3,msb")
spiPort.set_ssPolarity(YSpiPort.SSPOLARITY_ACTIVE_LOW)
spiPort.set_protocol("Frame:5ms")
spiPort.reset()

# do not forget to onfigure the powerOutput of the Yocto - SPI
# (for SPI7SEGDISP8.56 powerOutput need to be set at 5v)
print("****************************")
print("* make sure voltage levels *")
print("* are properly configured  *")
print("****************************")

# initialize MAX7219
spiPort.writeHex('0c01')  # Exit from shutdown state
spiPort.writeHex('09ff')  # Enable BCD for all digits
spiPort.writeHex('0b07')  # Enable digits 0-7 (=8 in total)
spiPort.writeHex('0a0a')  # Set medium brightness
for i in range(1, 9):
    digit = value % 10
    spiPort.writeArray([i, digit])
    value = int(value / 10)

YAPI.FreeAPI()



#!/usr/bin/python
# -*- coding: utf-8 -*-
import os, sys
import math

# add ../../Sources to the PYTHONPATH
sys.path.append(os.path.join("..", "..", "Sources"))
from array import *
from yoctopuce.yocto_api import *
from yoctopuce.yocto_files import *


def usage():
    scriptname = os.path.basename(sys.argv[0])
    print("Usage:")
    print(scriptname + ' <serial_number>')
    print(scriptname + ' <logical_name>')
    print(scriptname + ' any  ')
    sys.exit()


def die(msg):
    sys.exit(msg + ' (check USB cable)')


errmsg = YRefParam()

if len(sys.argv) < 2:
    usage()

target = sys.argv[1]

# Setup the API to use local USB devices
if YAPI.RegisterHub("usb", errmsg) != YAPI.SUCCESS:
    sys.exit("init error" + errmsg.value)

if target == 'any':
    # retrieve any filesystem
    files = YFiles.FirstFiles()
    if files is None:
        die('No module connected')
else:
    files = YFiles.FindFiles(target + ".files")

if not files.isOnline():
    die("Module not connected ")

# create text files and upload them to the device
for i in range(1, 5):
    contents = "This is file " + str(i)
    # convert the string to binary data
    binaryData = contents.encode("latin-1")
    # upload the file to the device
    files.upload("file" + str(i) + ".txt", binaryData)

# list files found on the device
print("Files on device:")
filelist = files.get_list("*")

for i in range(len(filelist)):
    file = filelist[i]
    print('%-40s%08x    %d bytes' % (file.get_name(), file.get_crc() % 0xffffffff, file.get_size()))

# download a file
binaryData = files.download("file1.txt")

# and display
print("")
print("contents of file1.txt:")
print(binaryData.decode("latin-1"))
YAPI.FreeAPI()


#*********************************************************************
#*
#* $Id: yocto_messagebox.py 24652 2016-05-31 13:47:30Z mvuilleu $
#*
#* Implements yFindMessageBox(), the high-level API for MessageBox functions
#*
#* - - - - - - - - - License information: - - - - - - - - -
#*
#*  Copyright (C) 2011 and beyond by Yoctopuce Sarl, Switzerland.
#*
#*  Yoctopuce Sarl (hereafter Licensor) grants to you a perpetual
#*  non-exclusive license to use, modify, copy and integrate this
#*  file into your software for the sole purpose of interfacing
#*  with Yoctopuce products.
#*
#*  You may reproduce and distribute copies of this file in
#*  source or object form, as long as the sole purpose of this
#*  code is to interface with Yoctopuce products. You must retain
#*  this notice in the distributed source file.
#*
#*  You should refer to Yoctopuce General Terms and Conditions
#*  for additional information regarding your rights and
#*  obligations.
#*
#*  THE SOFTWARE AND DOCUMENTATION ARE PROVIDED 'AS IS' WITHOUT
#*  WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING
#*  WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS
#*  FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO
#*  EVENT SHALL LICENSOR BE LIABLE FOR ANY INCIDENTAL, SPECIAL,
#*  INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA,
#*  COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR
#*  SERVICES, ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT NOT
#*  LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR
#*  CONTRIBUTION, OR OTHER SIMILAR COSTS, WHETHER ASSERTED ON THE
#*  BASIS OF CONTRACT, TORT (INCLUDING NEGLIGENCE), BREACH OF
#*  WARRANTY, OR OTHERWISE.
#*
#*********************************************************************/


__docformat__ = 'restructuredtext en'
from yoctopuce.yocto_api import *


#--- (generated code: YSms class start)
#noinspection PyProtectedMember
class YSms(object):
#--- (end of generated code: YSms class start)
    #--- (generated code: YSms return codes)
    #--- (end of generated code: YSms return codes)
    #--- (generated code: YSms dlldef)
    #--- (end of generated code: YSms dlldef)
    #--- (generated code: YSms definitions)
    #--- (end of generated code: YSms definitions)

    def __init__(self, obj_mbox):
        #--- (generated code: YSms attributes)
        self._mbox = None
        self._slot = 0
        self._deliv = 0
        self._smsc = ''
        self._mref = 0
        self._orig = ''
        self._dest = ''
        self._pid = 0
        self._alphab = 0
        self._mclass = 0
        self._stamp = ''
        self._udh = ''
        self._udata = ''
        self._npdu = 0
        self._pdu = ''
        self._parts = []
        self._aggSig = ''
        self._aggIdx = 0
        self._aggCnt = 0
        #--- (end of generated code: YSms attributes)
        self._mbox = obj_mbox

#--- (generated code: YSms implementation)
    def get_slot(self):
        return self._slot

    def get_smsc(self):
        return self._smsc

    def get_msgRef(self):
        return self._mref

    def get_sender(self):
        return self._orig

    def get_recipient(self):
        return self._dest

    def get_protocolId(self):
        return self._pid

    def isReceived(self):
        return self._deliv

    def get_alphabet(self):
        return self._alphab

    def get_msgClass(self):
        if ((self._mclass) & (16)) == 0:
            return -1
        return ((self._mclass) & (3))

    def get_dcs(self):
        return ((self._mclass) | ((((self._alphab) << (2)))))

    def get_timestamp(self):
        return self._stamp

    def get_userDataHeader(self):
        return self._udh

    def get_userData(self):
        return self._udata

    def get_textData(self):
        # isolatin
        # isosize
        # i
        
        if self._alphab == 0:
            #
            return self._mbox.gsm2str(self._udata)
        if self._alphab == 2:
            #
            isosize = ((len(self._udata)) >> (1))
            isolatin = bytearray(isosize)
            i = 0
            while i < isosize:
                isolatin[i] = YGetByte(self._udata, 2*i+1)
                i = i + 1
            return YByte2String(isolatin)
        
        # // default: convert 8 bit to string as-is
        return YByte2String(self._udata)

    def get_unicodeData(self):
        res = []
        # unisize
        # unival
        # i
        
        if self._alphab == 0:
            #
            return self._mbox.gsm2unicode(self._udata)
        if self._alphab == 2:
            #
            unisize = ((len(self._udata)) >> (1))
            del res[:]
            i = 0
            while i < unisize:
                unival = 256*YGetByte(self._udata, 2*i)+YGetByte(self._udata, 2*i+1)
                res.append(unival)
                i = i + 1
        else:
            #
            unisize = len(self._udata)
            del res[:]
            i = 0
            while i < unisize:
                res.append(YGetByte(self._udata, i)+0)
                i = i + 1
        return res

    def get_partCount(self):
        if self._npdu == 0:
            self.generatePdu()
        return self._npdu

    def get_pdu(self):
        if self._npdu == 0:
            self.generatePdu()
        return self._pdu

    def get_parts(self):
        if self._npdu == 0:
            self.generatePdu()
        return self._parts

    def get_concatSignature(self):
        if self._npdu == 0:
            self.generatePdu()
        return self._aggSig

    def get_concatIndex(self):
        if self._npdu == 0:
            self.generatePdu()
        return self._aggIdx

    def get_concatCount(self):
        if self._npdu == 0:
            self.generatePdu()
        return self._aggCnt

    def set_slot(self, val):
        self._slot = val
        return YAPI.SUCCESS

    def set_received(self, val):
        self._deliv = val
        return YAPI.SUCCESS

    def set_smsc(self, val):
        self._smsc = val
        self._npdu = 0
        return YAPI.SUCCESS

    def set_msgRef(self, val):
        self._mref = val
        self._npdu = 0
        return YAPI.SUCCESS

    def set_sender(self, val):
        self._orig = val
        self._npdu = 0
        return YAPI.SUCCESS

    def set_recipient(self, val):
        self._dest = val
        self._npdu = 0
        return YAPI.SUCCESS

    def set_protocolId(self, val):
        self._pid = val
        self._npdu = 0
        return YAPI.SUCCESS

    def set_alphabet(self, val):
        self._alphab = val
        self._npdu = 0
        return YAPI.SUCCESS

    def set_msgClass(self, val):
        if val == -1:
            self._mclass = 0
        else:
            self._mclass = 16+val
        self._npdu = 0
        return YAPI.SUCCESS

    def set_dcs(self, val):
        self._alphab = (((((val) >> (2)))) & (3))
        self._mclass = ((val) & (16+3))
        self._npdu = 0
        return YAPI.SUCCESS

    def set_timestamp(self, val):
        self._stamp = val
        self._npdu = 0
        return YAPI.SUCCESS

    def set_userDataHeader(self, val):
        self._udh = val
        self._npdu = 0
        self.parseUserDataHeader()
        return YAPI.SUCCESS

    def set_userData(self, val):
        self._udata = val
        self._npdu = 0
        return YAPI.SUCCESS

    def convertToUnicode(self):
        ucs2 = []
        # udatalen
        # i
        # uni
        
        if self._alphab == 2:
            return YAPI.SUCCESS
        if self._alphab == 0:
            ucs2 = self._mbox.gsm2unicode(self._udata)
        else:
            udatalen = len(self._udata)
            del ucs2[:]
            i = 0
            while i < udatalen:
                uni = YGetByte(self._udata, i)
                ucs2.append(uni)
                i = i + 1
        self._alphab = 2
        self._udata = bytearray(0)
        self.addUnicodeData(ucs2)
        
        return YAPI.SUCCESS

    def addText(self, val):
        # udata
        # udatalen
        # newdata
        # newdatalen
        # i
        
        if len(val) == 0:
            return YAPI.SUCCESS
        
        if self._alphab == 0:
            #
            newdata = self._mbox.str2gsm(val)
            newdatalen = len(newdata)
            if newdatalen == 0:
                #
                self.convertToUnicode()
                newdata = YString2Byte(val)
                newdatalen = len(newdata)
        else:
            newdata = YString2Byte(val)
            newdatalen = len(newdata)
        udatalen = len(self._udata)
        if self._alphab == 2:
            #
            udata = bytearray(udatalen + 2*newdatalen)
            i = 0
            while i < udatalen:
                udata[i] = YGetByte(self._udata, i)
                i = i + 1
            i = 0
            while i < newdatalen:
                udata[udatalen+1] = YGetByte(newdata, i)
                udatalen = udatalen + 2
                i = i + 1
        else:
            #
            udata = bytearray(udatalen+newdatalen)
            i = 0
            while i < udatalen:
                udata[i] = YGetByte(self._udata, i)
                i = i + 1
            i = 0
            while i < newdatalen:
                udata[udatalen] = YGetByte(newdata, i)
                udatalen = udatalen + 1
                i = i + 1
        
        return self.set_userData(udata)

    def addUnicodeData(self, val):
        # arrlen
        # newdatalen
        # i
        # uni
        # udata
        # udatalen
        # surrogate
        
        if self._alphab != 2:
            self.convertToUnicode()
        # // compute number of 16-bit code units
        arrlen = len(val)
        newdatalen = arrlen
        i = 0
        while i < arrlen:
            uni = val[i]
            if uni > 65535:
                newdatalen = newdatalen + 1
            i = i + 1
        # // now build utf-16 buffer
        udatalen = len(self._udata)
        udata = bytearray(udatalen+2*newdatalen)
        i = 0
        while i < udatalen:
            udata[i] = YGetByte(self._udata, i)
            i = i + 1
        i = 0
        while i < arrlen:
            uni = val[i]
            if uni >= 65536:
                surrogate = uni - 65536
                uni = (((((surrogate) >> (10))) & (1023))) + 55296
                udata[udatalen] = ((uni) >> (8))
                udata[udatalen+1] = ((uni) & (255))
                udatalen = udatalen + 2
                uni = (((surrogate) & (1023))) + 56320
            udata[udatalen] = ((uni) >> (8))
            udata[udatalen+1] = ((uni) & (255))
            udatalen = udatalen + 2
            i = i + 1
        
        return self.set_userData(udata)

    def set_pdu(self, pdu):
        self._pdu = pdu
        self._npdu = 1
        return self.parsePdu(pdu)

    def set_parts(self, parts):
        sorted = []
        # partno
        # initpartno
        # i
        # retcode
        # totsize
        # subsms
        # subdata
        # res
        self._npdu = len(parts)
        if self._npdu == 0:
            return YAPI.INVALID_ARGUMENT
        
        del sorted[:]
        partno = 0
        while partno < self._npdu:
            initpartno = partno
            i = 0
            while i < self._npdu:
                subsms = parts[i]
                if subsms.get_concatIndex() == partno:
                    sorted.append(subsms)
                    partno = partno + 1
                i = i + 1
            if initpartno == partno:
                partno = partno + 1
        
        self._parts = sorted
        self._npdu = len(sorted)
        # // inherit header fields from first part
        subsms = self._parts[0]
        retcode = self.parsePdu(subsms.get_pdu())
        if retcode != YAPI.SUCCESS:
            return retcode
        # // concatenate user data from all parts
        totsize = 0
        partno = 0
        while partno < len(self._parts):
            subsms = self._parts[partno]
            subdata = subsms.get_userData()
            totsize = totsize + len(subdata)
            partno = partno + 1
        res = bytearray(totsize)
        totsize = 0
        partno = 0
        while partno < len(self._parts):
            subsms = self._parts[partno]
            subdata = subsms.get_userData()
            i = 0
            while i < len(subdata):
                res[totsize] = YGetByte(subdata, i)
                totsize = totsize + 1
                i = i + 1
            partno = partno + 1
        self._udata = res
        return YAPI.SUCCESS

    def encodeAddress(self, addr):
        # bytes
        # srclen
        # numlen
        # i
        # val
        # digit
        # res
        bytes = YString2Byte(addr)
        srclen = len(bytes)
        numlen = 0
        i = 0
        while i < srclen:
            val = YGetByte(bytes, i)
            if (val >= 48) and (val < 58):
                numlen = numlen + 1
            i = i + 1
        if numlen == 0:
            res = bytearray(1)
            res[0] = 0
            return res
        res = bytearray(2+((numlen+1) >> (1)))
        res[0] = numlen
        if YGetByte(bytes, 0) == 43:
            res[1] = 145
        else:
            res[1] = 129
        numlen = 4
        digit = 0
        i = 0
        while i < srclen:
            val = YGetByte(bytes, i)
            if (val >= 48) and (val < 58):
                if ((numlen) & (1)) == 0:
                    digit = val - 48
                else:
                    res[((numlen) >> (1))] = digit + 16*(val-48)
                numlen = numlen + 1
            i = i + 1
        # // pad with F if needed
        if ((numlen) & (1)) != 0:
            res[((numlen) >> (1))] = digit + 240
        return res

    def decodeAddress(self, addr, ofs, siz):
        # addrType
        # gsm7
        # res
        # i
        # rpos
        # carry
        # nbits
        # byt
        if siz == 0:
            return ""
        res = ""
        addrType = ((YGetByte(addr, ofs)) & (112))
        if addrType == 80:
            #
            siz = int((4*siz) / (7))
            gsm7 = bytearray(siz)
            rpos = 1
            carry = 0
            nbits = 0
            i = 0
            while i < siz:
                if nbits == 7:
                    gsm7[i] = carry
                    carry = 0
                    nbits = 0
                else:
                    byt = YGetByte(addr, ofs+rpos)
                    rpos = rpos + 1
                    gsm7[i] = ((carry) | ((((((byt) << (nbits)))) & (127))))
                    carry = ((byt) >> ((7 - nbits)))
                    nbits = nbits + 1
                i = i + 1
            return self._mbox.gsm2str(gsm7)
        else:
            #
            if addrType == 16:
                res = "+"
            siz = (((siz+1)) >> (1))
            i = 0
            while i < siz:
                byt = YGetByte(addr, ofs+i+1)
                res = "" + res + "" + ("%X" % ((byt) & (15))) + "" + ("%X" % ((byt) >> (4)))
                i = i + 1
            #
            if ((YGetByte(addr, ofs+siz)) >> (4)) == 15:
                res = (res)[0: 0 + len(res)-1]
            return res

    def encodeTimeStamp(self, exp):
        # explen
        # i
        # res
        # n
        # expasc
        # v1
        # v2
        explen = len(exp)
        if explen == 0:
            res = bytearray(0)
            return res
        if (exp)[0: 0 + 1] == "+":
            n = YAPI._atoi((exp)[1: 1 + explen-1])
            res = bytearray(1)
            if n > 30*86400:
                n = 192+int(((n+6*86400)) / ((7*86400)))
            else:
                if n > 86400:
                    n = 166+int(((n+86399)) / (86400))
                else:
                    if n > 43200:
                        n = 143+int(((n-43200+1799)) / (1800))
                    else:
                        n = -1+int(((n+299)) / (300))
            if n < 0:
                n = 0
            res[0] = n
            return res
        if (exp)[4: 4 + 1] == "-" or (exp)[4: 4 + 1] == "/":
            #
            exp = (exp)[2: 2 + explen-2]
            explen = len(exp)
        expasc = YString2Byte(exp)
        res = bytearray(7)
        n = 0
        i = 0
        while (i+1 < explen) and (n < 7):
            v1 = YGetByte(expasc, i)
            if (v1 >= 48) and (v1 < 58):
                v2 = YGetByte(expasc, i+1)
                if (v2 >= 48) and (v2 < 58):
                    v1 = v1 - 48
                    v2 = v2 - 48
                    res[n] = (((v2) << (4))) + v1
                    n = n + 1
                    i = i + 1
            i = i + 1
        while n < 7:
            res[n] = 0
            n = n + 1
        if i+2 < explen:
            #
            v1 = YGetByte(expasc, i-3)
            v2 = YGetByte(expasc, i)
            if ((v1 == 43) or (v1 == 45)) and (v2 == 58):
                v1 = YGetByte(expasc, i+1)
                v2 = YGetByte(expasc, i+2)
                if (v1 >= 48) and (v1 < 58) and (v1 >= 48) and (v1 < 58):
                    v1 = int(((10*(v1 - 48)+(v2 - 48))) / (15))
                    n = n - 1
                    v2 = 4 * YGetByte(res, n) + v1
                    if YGetByte(expasc, i-3) == 45:
                        v2 += 128
                    res[n] = v2
        return res

    def decodeTimeStamp(self, exp, ofs, siz):
        # n
        # res
        # i
        # byt
        # sign
        # hh
        # ss
        if siz < 1:
            return ""
        if siz == 1:
            n = YGetByte(exp, ofs)
            if n < 144:
                n = n * 300
            else:
                if n < 168:
                    n = (n-143) * 1800
                else:
                    if n < 197:
                        n = (n-166) * 86400
                    else:
                        n = (n-192) * 7 * 86400
            return "+" + str(int(n))
        res = "20"
        i = 0
        while (i < siz) and (i < 6):
            byt = YGetByte(exp, ofs+i)
            res = "" + res + "" + ("%X" % ((byt) & (15))) + "" + ("%X" % ((byt) >> (4)))
            if i < 3:
                if i < 2:
                    res = "" + res + "-"
                else:
                    res = "" + res + " "
            else:
                if i < 5:
                    res = "" + res + ":"
            i = i + 1
        if siz == 7:
            byt = YGetByte(exp, ofs+i)
            sign = "+"
            if ((byt) & (8)) != 0:
                byt = byt - 8
                sign = "-"
            byt = (10*(((byt) & (15)))) + (((byt) >> (4)))
            hh = "" + str(int(((byt) >> (2))))
            ss = "" + str(int(15*(((byt) & (3)))))
            if len(hh)<2:
                hh = "0" + hh
            if len(ss)<2:
                ss = "0" + ss
            res = "" + res + "" + sign + "" + hh + ":" + ss
        return res

    def udataSize(self):
        # res
        # udhsize
        udhsize = len(self._udh)
        res = len(self._udata)
        if self._alphab == 0:
            if udhsize > 0:
                res = res + int(((8 + 8*udhsize + 6)) / (7))
            res = int(((res * 7 + 7)) / (8))
        else:
            if udhsize > 0:
                res = res + 1 + udhsize
        return res

    def encodeUserData(self):
        # udsize
        # udlen
        # udhsize
        # udhlen
        # res
        # i
        # wpos
        # carry
        # nbits
        # thisb
        # // nbits = number of bits in carry
        udsize = self.udataSize()
        udhsize = len(self._udh)
        udlen = len(self._udata)
        res = bytearray(1+udsize)
        udhlen = 0
        nbits = 0
        carry = 0
        # // 1. Encode UDL
        if self._alphab == 0:
            #
            if udhsize > 0:
                udhlen = int(((8 + 8*udhsize + 6)) / (7))
                nbits = 7*udhlen - 8 - 8*udhsize
            res[0] = udhlen+udlen
        else:
            #
            res[0] = udsize
        # // 2. Encode UDHL and UDL
        wpos = 1
        if udhsize > 0:
            res[wpos] = udhsize
            wpos = wpos + 1
            i = 0
            while i < udhsize:
                res[wpos] = YGetByte(self._udh, i)
                wpos = wpos + 1
                i = i + 1
        # // 3. Encode UD
        if self._alphab == 0:
            #
            i = 0
            while i < udlen:
                if nbits == 0:
                    carry = YGetByte(self._udata, i)
                    nbits = 7
                else:
                    thisb = YGetByte(self._udata, i)
                    res[wpos] = ((carry) | ((((((thisb) << (nbits)))) & (255))))
                    wpos = wpos + 1
                    nbits = nbits - 1
                    carry = ((thisb) >> ((7 - nbits)))
                i = i + 1
            if nbits > 0:
                res[wpos] = carry
        else:
            #
            i = 0
            while i < udlen:
                res[wpos] = YGetByte(self._udata, i)
                wpos = wpos + 1
                i = i + 1
        return res

    def generateParts(self):
        # udhsize
        # udlen
        # mss
        # partno
        # partlen
        # newud
        # newudh
        # newpdu
        # i
        # wpos
        udhsize = len(self._udh)
        udlen = len(self._udata)
        mss = 140 - 1 - 5 - udhsize
        if self._alphab == 0:
            mss = int(((mss * 8 - 6)) / (7))
        self._npdu = int(((udlen+mss-1)) / (mss))
        del self._parts[:]
        partno = 0
        wpos = 0
        while wpos < udlen:
            partno = partno + 1
            newudh = bytearray(5+udhsize)
            newudh[0] = 0
            #
            newudh[1] = 3
            #
            newudh[2] = self._mref
            newudh[3] = self._npdu
            newudh[4] = partno
            i = 0
            while i < udhsize:
                newudh[5+i] = YGetByte(self._udh, i)
                i = i + 1
            if wpos+mss < udlen:
                partlen = mss
            else:
                partlen = udlen-wpos
            newud = bytearray(partlen)
            i = 0
            while i < partlen:
                newud[i] = YGetByte(self._udata, wpos)
                wpos = wpos + 1
                i = i + 1
            newpdu = YSms(self._mbox)
            newpdu.set_received(self.isReceived())
            newpdu.set_smsc(self.get_smsc())
            newpdu.set_msgRef(self.get_msgRef())
            newpdu.set_sender(self.get_sender())
            newpdu.set_recipient(self.get_recipient())
            newpdu.set_protocolId(self.get_protocolId())
            newpdu.set_dcs(self.get_dcs())
            newpdu.set_timestamp(self.get_timestamp())
            newpdu.set_userDataHeader(newudh)
            newpdu.set_userData(newud)
            self._parts.append(newpdu)
        return YAPI.SUCCESS

    def generatePdu(self):
        # sca
        # hdr
        # addr
        # stamp
        # udata
        # pdutyp
        # pdulen
        # i
        # // Determine if the message can fit within a single PDU
        del self._parts[:]
        if self.udataSize() > 140:
            #
            self._pdu = bytearray(0)
            return self.generateParts()
        sca = self.encodeAddress(self._smsc)
        if len(sca) > 0:
            sca[0] = len(sca)-1
        stamp = self.encodeTimeStamp(self._stamp)
        udata = self.encodeUserData()
        if self._deliv:
            addr = self.encodeAddress(self._orig)
            hdr = bytearray(1)
            pdutyp = 0
        else:
            addr = self.encodeAddress(self._dest)
            self._mref = self._mbox.nextMsgRef()
            hdr = bytearray(2)
            hdr[1] = self._mref
            pdutyp = 1
            if len(stamp) > 0:
                pdutyp = pdutyp + 16
            if len(stamp) == 7:
                pdutyp = pdutyp + 8
        if len(self._udh) > 0:
            pdutyp = pdutyp + 64
        hdr[0] = pdutyp
        pdulen = len(sca)+len(hdr)+len(addr)+2+len(stamp)+len(udata)
        self._pdu = bytearray(pdulen)
        pdulen = 0
        i = 0
        while i < len(sca):
            self._pdu[pdulen] = YGetByte(sca, i)
            pdulen = pdulen + 1
            i = i + 1
        i = 0
        while i < len(hdr):
            self._pdu[pdulen] = YGetByte(hdr, i)
            pdulen = pdulen + 1
            i = i + 1
        i = 0
        while i < len(addr):
            self._pdu[pdulen] = YGetByte(addr, i)
            pdulen = pdulen + 1
            i = i + 1
        self._pdu[pdulen] = self._pid
        pdulen = pdulen + 1
        self._pdu[pdulen] = self.get_dcs()
        pdulen = pdulen + 1
        i = 0
        while i < len(stamp):
            self._pdu[pdulen] = YGetByte(stamp, i)
            pdulen = pdulen + 1
            i = i + 1
        i = 0
        while i < len(udata):
            self._pdu[pdulen] = YGetByte(udata, i)
            pdulen = pdulen + 1
            i = i + 1
        self._npdu = 1
        return YAPI.SUCCESS

    def parseUserDataHeader(self):
        # udhlen
        # i
        # iei
        # ielen
        # sig
        
        self._aggSig = ""
        self._aggIdx = 0
        self._aggCnt = 0
        udhlen = len(self._udh)
        i = 0
        while i+1 < udhlen:
            iei = YGetByte(self._udh, i)
            ielen = YGetByte(self._udh, i+1)
            i = i + 2
            if i + ielen <= udhlen:
                if (iei == 0) and (ielen == 3):
                    #
                    sig = "" + self._orig + "-" + self._dest + "-" + ("%02X" % self._mref) + "-" + ("%02X" % YGetByte(self._udh, i))
                    self._aggSig = sig
                    self._aggCnt = YGetByte(self._udh, i+1)
                    self._aggIdx = YGetByte(self._udh, i+2)
                if (iei == 8) and (ielen == 4):
                    #
                    sig = "" + self._orig + "-" + self._dest + "-" + ("%02X" % self._mref) + "-" + ("%02X" % YGetByte(self._udh, i)) + "" + ("%02X" % YGetByte(self._udh, i+1))
                    self._aggSig = sig
                    self._aggCnt = YGetByte(self._udh, i+2)
                    self._aggIdx = YGetByte(self._udh, i+3)
            i = i + ielen
        return YAPI.SUCCESS

    def parsePdu(self, pdu):
        # rpos
        # addrlen
        # pdutyp
        # tslen
        # dcs
        # udlen
        # udhsize
        # udhlen
        # i
        # carry
        # nbits
        # thisb
        
        self._pdu = pdu
        self._npdu = 1
        
        # // parse meta-data
        self._smsc = self.decodeAddress(pdu, 1, 2*(YGetByte(pdu, 0)-1))
        rpos = 1+YGetByte(pdu, 0)
        pdutyp = YGetByte(pdu, rpos)
        rpos = rpos + 1
        self._deliv = (((pdutyp) & (3)) == 0)
        if self._deliv:
            addrlen = YGetByte(pdu, rpos)
            rpos = rpos + 1
            self._orig = self.decodeAddress(pdu, rpos, addrlen)
            self._dest = ""
            tslen = 7
        else:
            self._mref = YGetByte(pdu, rpos)
            rpos = rpos + 1
            addrlen = YGetByte(pdu, rpos)
            rpos = rpos + 1
            self._dest = self.decodeAddress(pdu, rpos, addrlen)
            self._orig = ""
            if (((pdutyp) & (16))) != 0:
                if (((pdutyp) & (8))) != 0:
                    tslen = 7
                else:
                    tslen= 1
            else:
                tslen = 0
        rpos = rpos + ((((addrlen+3)) >> (1)))
        self._pid = YGetByte(pdu, rpos)
        rpos = rpos + 1
        dcs = YGetByte(pdu, rpos)
        rpos = rpos + 1
        self._alphab = (((((dcs) >> (2)))) & (3))
        self._mclass = ((dcs) & (16+3))
        self._stamp = self.decodeTimeStamp(pdu, rpos, tslen)
        rpos = rpos + tslen
        
        # // parse user data (including udh)
        nbits = 0
        carry = 0
        udlen = YGetByte(pdu, rpos)
        rpos = rpos + 1
        if ((pdutyp) & (64)) != 0:
            udhsize = YGetByte(pdu, rpos)
            rpos = rpos + 1
            self._udh = bytearray(udhsize)
            i = 0
            while i < udhsize:
                self._udh[i] = YGetByte(pdu, rpos)
                rpos = rpos + 1
                i = i + 1
            if self._alphab == 0:
                #
                udhlen = int(((8 + 8*udhsize + 6)) / (7))
                nbits = 7*udhlen - 8 - 8*udhsize
                if nbits > 0:
                    thisb = YGetByte(pdu, rpos)
                    rpos = rpos + 1
                    carry = ((thisb) >> (nbits))
                    nbits = 8 - nbits
            else:
                #
                udhlen = 1+udhsize
            udlen = udlen - udhlen
        else:
            udhsize = 0
            self._udh = bytearray(0)
        self._udata = bytearray(udlen)
        if self._alphab == 0:
            #
            i = 0
            while i < udlen:
                if nbits == 7:
                    self._udata[i] = carry
                    carry = 0
                    nbits = 0
                else:
                    thisb = YGetByte(pdu, rpos)
                    rpos = rpos + 1
                    self._udata[i] = ((carry) | ((((((thisb) << (nbits)))) & (127))))
                    carry = ((thisb) >> ((7 - nbits)))
                    nbits = nbits + 1
                i = i + 1
        else:
            #
            i = 0
            while i < udlen:
                self._udata[i] = YGetByte(pdu, rpos)
                rpos = rpos + 1
                i = i + 1
        self.parseUserDataHeader()
        
        return YAPI.SUCCESS

    def send(self):
        # i
        # retcode
        # pdu
        # // may throw an exception
        if self._npdu == 0:
            self.generatePdu()
        if self._npdu == 1:
            return self._mbox._upload("sendSMS", self._pdu)
        retcode = YAPI.SUCCESS
        i = 0
        while (i < self._npdu) and (retcode == YAPI.SUCCESS):
            pdu = self._parts[i]
            retcode= pdu.send()
            i = i + 1
        return retcode

    def deleteFromSIM(self):
        # i
        # retcode
        # pdu
        # // may throw an exception
        if self._slot > 0:
            return self._mbox.clearSIMSlot(self._slot)
        retcode = YAPI.SUCCESS
        i = 0
        while (i < self._npdu) and (retcode == YAPI.SUCCESS):
            pdu = self._parts[i]
            retcode= pdu.deleteFromSIM()
            i = i + 1
        return retcode

#--- (end of generated code: YSms implementation)

#--- (generated code: Sms functions)
#--- (end of generated code: Sms functions)


#--- (generated code: YMessageBox class start)
#noinspection PyProtectedMember
class YMessageBox(YFunction):
    """
    YMessageBox functions provides SMS sending and receiving capability to
    GSM-enabled Yoctopuce devices.

    """
#--- (end of generated code: YMessageBox class start)
    #--- (generated code: YMessageBox return codes)
    #--- (end of generated code: YMessageBox return codes)
    #--- (generated code: YMessageBox dlldef)
    #--- (end of generated code: YMessageBox dlldef)
    #--- (generated code: YMessageBox definitions)
    SLOTSINUSE_INVALID = YAPI.INVALID_UINT
    SLOTSCOUNT_INVALID = YAPI.INVALID_UINT
    SLOTSBITMAP_INVALID = YAPI.INVALID_STRING
    PDUSENT_INVALID = YAPI.INVALID_UINT
    PDURECEIVED_INVALID = YAPI.INVALID_UINT
    COMMAND_INVALID = YAPI.INVALID_STRING
    #--- (end of generated code: YMessageBox definitions)

    def __init__(self, func):
        super(YMessageBox, self).__init__(func)
        self._className = 'MessageBox'
        #--- (generated code: YMessageBox attributes)
        self._callback = None
        self._slotsInUse = YMessageBox.SLOTSINUSE_INVALID
        self._slotsCount = YMessageBox.SLOTSCOUNT_INVALID
        self._slotsBitmap = YMessageBox.SLOTSBITMAP_INVALID
        self._pduSent = YMessageBox.PDUSENT_INVALID
        self._pduReceived = YMessageBox.PDURECEIVED_INVALID
        self._command = YMessageBox.COMMAND_INVALID
        self._nextMsgRef = 0
        self._prevBitmapStr = ''
        self._pdus = []
        self._messages = []
        self._gsm2unicodeReady = 0
        self._gsm2unicode = []
        self._iso2gsm = ''
        #--- (end of generated code: YMessageBox attributes)

    #--- (generated code: YMessageBox implementation)
    def _parseAttr(self, member):
        if member.name == "slotsInUse":
            self._slotsInUse = member.ivalue
            return 1
        if member.name == "slotsCount":
            self._slotsCount = member.ivalue
            return 1
        if member.name == "slotsBitmap":
            self._slotsBitmap = member.svalue
            return 1
        if member.name == "pduSent":
            self._pduSent = member.ivalue
            return 1
        if member.name == "pduReceived":
            self._pduReceived = member.ivalue
            return 1
        if member.name == "command":
            self._command = member.svalue
            return 1
        super(YMessageBox, self)._parseAttr(member)

    def get_slotsInUse(self):
        """
        Returns the number of message storage slots currently in use.

        @return an integer corresponding to the number of message storage slots currently in use

        On failure, throws an exception or returns YMessageBox.SLOTSINUSE_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YMessageBox.SLOTSINUSE_INVALID
        return self._slotsInUse

    def get_slotsCount(self):
        """
        Returns the total number of message storage slots on the SIM card.

        @return an integer corresponding to the total number of message storage slots on the SIM card

        On failure, throws an exception or returns YMessageBox.SLOTSCOUNT_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YMessageBox.SLOTSCOUNT_INVALID
        return self._slotsCount

    def get_slotsBitmap(self):
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YMessageBox.SLOTSBITMAP_INVALID
        return self._slotsBitmap

    def get_pduSent(self):
        """
        Returns the number of SMS units sent so far.

        @return an integer corresponding to the number of SMS units sent so far

        On failure, throws an exception or returns YMessageBox.PDUSENT_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YMessageBox.PDUSENT_INVALID
        return self._pduSent

    def set_pduSent(self, newval):
        """
        Changes the value of the outgoing SMS units counter.

        @param newval : an integer corresponding to the value of the outgoing SMS units counter

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return self._setAttr("pduSent", rest_val)

    def get_pduReceived(self):
        """
        Returns the number of SMS units received so far.

        @return an integer corresponding to the number of SMS units received so far

        On failure, throws an exception or returns YMessageBox.PDURECEIVED_INVALID.
        """
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YMessageBox.PDURECEIVED_INVALID
        return self._pduReceived

    def set_pduReceived(self, newval):
        """
        Changes the value of the incoming SMS units counter.

        @param newval : an integer corresponding to the value of the incoming SMS units counter

        @return YAPI.SUCCESS if the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        rest_val = str(newval)
        return self._setAttr("pduReceived", rest_val)

    def get_command(self):
        if self._cacheExpiration <= YAPI.GetTickCount():
            if self.load(YAPI.DefaultCacheValidity) != YAPI.SUCCESS:
                return YMessageBox.COMMAND_INVALID
        return self._command

    def set_command(self, newval):
        rest_val = newval
        return self._setAttr("command", rest_val)

    @staticmethod
    def FindMessageBox(func):
        """
        Retrieves a MessageBox interface for a given identifier.
        The identifier can be specified using several formats:
        <ul>
        <li>FunctionLogicalName</li>
        <li>ModuleSerialNumber.FunctionIdentifier</li>
        <li>ModuleSerialNumber.FunctionLogicalName</li>
        <li>ModuleLogicalName.FunctionIdentifier</li>
        <li>ModuleLogicalName.FunctionLogicalName</li>
        </ul>

        This function does not require that the MessageBox interface is online at the time
        it is invoked. The returned object is nevertheless valid.
        Use the method YMessageBox.isOnline() to test if the MessageBox interface is
        indeed online at a given time. In case of ambiguity when looking for
        a MessageBox interface by logical name, no error is notified: the first instance
        found is returned. The search is performed first by hardware name,
        then by logical name.

        @param func : a string that uniquely characterizes the MessageBox interface

        @return a YMessageBox object allowing you to drive the MessageBox interface.
        """
        # obj
        obj = YFunction._FindFromCache("MessageBox", func)
        if obj is None:
            obj = YMessageBox(func)
            YFunction._AddToCache("MessageBox", func, obj)
        return obj

    def nextMsgRef(self):
        self._nextMsgRef = self._nextMsgRef + 1
        return self._nextMsgRef

    def clearSIMSlot(self, slot):
        # // may throw an exception
        self._prevBitmapStr = ""
        return self.set_command("DS" + str(int(slot)))

    def fetchPdu(self, slot):
        # binPdu
        arrPdu = []
        # hexPdu
        # sms
        
        # // may throw an exception
        binPdu = self._download("sms.json?pos=" + str(int(slot)) + "&len=1")
        arrPdu = self._json_get_array(binPdu)
        hexPdu = self._decode_json_string(arrPdu[0])
        sms = YSms(self)
        sms.set_slot(slot)
        sms.parsePdu(YAPI._hexStrToBin(hexPdu))
        return sms

    def initGsm2Unicode(self):
        # i
        # uni
        
        del self._gsm2unicode[:]
        # // 00-07
        self._gsm2unicode.append(64)
        self._gsm2unicode.append(163)
        self._gsm2unicode.append(36)
        self._gsm2unicode.append(165)
        self._gsm2unicode.append(232)
        self._gsm2unicode.append(233)
        self._gsm2unicode.append(249)
        self._gsm2unicode.append(236)
        # // 08-0F
        self._gsm2unicode.append(242)
        self._gsm2unicode.append(199)
        self._gsm2unicode.append(10)
        self._gsm2unicode.append(216)
        self._gsm2unicode.append(248)
        self._gsm2unicode.append(13)
        self._gsm2unicode.append(197)
        self._gsm2unicode.append(229)
        # // 10-17
        self._gsm2unicode.append(916)
        self._gsm2unicode.append(95)
        self._gsm2unicode.append(934)
        self._gsm2unicode.append(915)
        self._gsm2unicode.append(923)
        self._gsm2unicode.append(937)
        self._gsm2unicode.append(928)
        self._gsm2unicode.append(936)
        # // 18-1F
        self._gsm2unicode.append(931)
        self._gsm2unicode.append(920)
        self._gsm2unicode.append(926)
        self._gsm2unicode.append(27)
        self._gsm2unicode.append(198)
        self._gsm2unicode.append(230)
        self._gsm2unicode.append(223)
        self._gsm2unicode.append(201)
        # // 20-7A
        i = 32
        while i <= 122:
            self._gsm2unicode.append(i)
            i = i + 1
        # // exceptions in range 20-7A
        self._gsm2unicode[36] = 164
        self._gsm2unicode[64] = 161
        self._gsm2unicode[91] = 196
        self._gsm2unicode[92] = 214
        self._gsm2unicode[93] = 209
        self._gsm2unicode[94] = 220
        self._gsm2unicode[95] = 167
        self._gsm2unicode[96] = 191
        # // 7B-7F
        self._gsm2unicode.append(228)
        self._gsm2unicode.append(246)
        self._gsm2unicode.append(241)
        self._gsm2unicode.append(252)
        self._gsm2unicode.append(224)
        
        # // Invert table as well wherever possible
        self._iso2gsm = bytearray(256)
        i = 0
        while i <= 127:
            uni = self._gsm2unicode[i]
            if uni <= 255:
                self._iso2gsm[uni] = i
            i = i + 1
        i = 0
        while i < 4:
            #
            self._iso2gsm[91+i] = 27
            self._iso2gsm[123+i] = 27
            i = i + 1
        # // Done
        self._gsm2unicodeReady = True
        
        return YAPI.SUCCESS

    def gsm2unicode(self, gsm):
        # i
        # gsmlen
        # reslen
        res = []
        # uni
        
        if not (self._gsm2unicodeReady):
            self.initGsm2Unicode()
        gsmlen = len(gsm)
        reslen = gsmlen
        i = 0
        while i < gsmlen:
            if YGetByte(gsm, i) == 27:
                reslen = reslen - 1
            i = i + 1
        del res[:]
        i = 0
        while i < gsmlen:
            uni = self._gsm2unicode[YGetByte(gsm, i)]
            if (uni == 27) and (i+1 < gsmlen):
                i = i + 1
                uni = YGetByte(gsm, i)
                if uni < 60:
                    if uni < 41:
                        if uni==20:
                            uni=94
                        else:
                            if uni==40:
                                uni=123
                            else:
                                uni=0
                    else:
                        if uni==41:
                            uni=125
                        else:
                            if uni==47:
                                uni=92
                            else:
                                uni=0
                else:
                    if uni < 62:
                        if uni==60:
                            uni=91
                        else:
                            if uni==61:
                                uni=126
                            else:
                                uni=0
                    else:
                        if uni==62:
                            uni=93
                        else:
                            if uni==64:
                                uni=124
                            else:
                                if uni==101:
                                    uni=164
                                else:
                                    uni=0
            if uni > 0:
                res.append(uni)
            i = i + 1
        
        return res

    def gsm2str(self, gsm):
        # i
        # gsmlen
        # reslen
        # resbin
        # resstr
        # uni
        
        if not (self._gsm2unicodeReady):
            self.initGsm2Unicode()
        gsmlen = len(gsm)
        reslen = gsmlen
        i = 0
        while i < gsmlen:
            if YGetByte(gsm, i) == 27:
                reslen = reslen - 1
            i = i + 1
        resbin = bytearray(reslen)
        i = 0
        reslen = 0
        while i < gsmlen:
            uni = self._gsm2unicode[YGetByte(gsm, i)]
            if (uni == 27) and (i+1 < gsmlen):
                i = i + 1
                uni = YGetByte(gsm, i)
                if uni < 60:
                    if uni < 41:
                        if uni==20:
                            uni=94
                        else:
                            if uni==40:
                                uni=123
                            else:
                                uni=0
                    else:
                        if uni==41:
                            uni=125
                        else:
                            if uni==47:
                                uni=92
                            else:
                                uni=0
                else:
                    if uni < 62:
                        if uni==60:
                            uni=91
                        else:
                            if uni==61:
                                uni=126
                            else:
                                uni=0
                    else:
                        if uni==62:
                            uni=93
                        else:
                            if uni==64:
                                uni=124
                            else:
                                if uni==101:
                                    uni=164
                                else:
                                    uni=0
            if (uni > 0) and (uni < 256):
                resbin[reslen] = uni
                reslen = reslen + 1
            i = i + 1
        resstr = YByte2String(resbin)
        if len(resstr) > reslen:
            resstr = (resstr)[0: 0 + reslen]
        return resstr

    def str2gsm(self, msg):
        # asc
        # asclen
        # i
        # ch
        # gsm7
        # extra
        # res
        # wpos
        
        if not (self._gsm2unicodeReady):
            self.initGsm2Unicode()
        asc = YString2Byte(msg)
        asclen = len(asc)
        extra = 0
        i = 0
        while i < asclen:
            ch = YGetByte(asc, i)
            gsm7 = YGetByte(self._iso2gsm, ch)
            if gsm7 == 27:
                extra = extra + 1
            if gsm7 == 0:
                #
                res = bytearray(0)
                return res
            i = i + 1
        res = bytearray(asclen+extra)
        wpos = 0
        i = 0
        while i < asclen:
            ch = YGetByte(asc, i)
            gsm7 = YGetByte(self._iso2gsm, ch)
            res[wpos] = gsm7
            wpos = wpos + 1
            if gsm7 == 27:
                if ch < 100:
                    if ch<93:
                        if ch<92:
                            gsm7=60
                        else:
                            gsm7=47
                    else:
                        if ch<94:
                            gsm7=62
                        else:
                            gsm7=20
                else:
                    if ch<125:
                        if ch<124:
                            gsm7=40
                        else:
                            gsm7=64
                    else:
                        if ch<126:
                            gsm7=41
                        else:
                            gsm7=61
                res[wpos] = gsm7
                wpos = wpos + 1
            i = i + 1
        return res

    def checkNewMessages(self):
        # bitmapStr
        # prevBitmap
        # newBitmap
        # slot
        # nslots
        # pduIdx
        # idx
        # bitVal
        # prevBit
        # i
        # nsig
        # cnt
        # sig
        newArr = []
        newMsg = []
        newAgg = []
        signatures = []
        # sms
        
        # // may throw an exception
        bitmapStr = self.get_slotsBitmap()
        if bitmapStr == self._prevBitmapStr:
            return YAPI.SUCCESS
        prevBitmap = YAPI._hexStrToBin(self._prevBitmapStr)
        newBitmap = YAPI._hexStrToBin(bitmapStr)
        self._prevBitmapStr = bitmapStr
        nslots = 8*len(newBitmap)
        del newArr[:]
        del newMsg[:]
        del signatures[:]
        nsig = 0
        # // copy known messages
        pduIdx = 0
        while pduIdx < len(self._pdus):
            sms = self._pdus[pduIdx]
            slot = sms.get_slot()
            idx = ((slot) >> (3))
            if idx < len(newBitmap):
                bitVal = ((1) << ((((slot) & (7)))))
                if (((YGetByte(newBitmap, idx)) & (bitVal))) != 0:
                    newArr.append(sms)
                    if sms.get_concatCount() == 0:
                        newMsg.append(sms)
                    else:
                        sig = sms.get_concatSignature()
                        i = 0
                        while (i < nsig) and (len(sig) > 0):
                            if signatures[i] == sig:
                                sig = ""
                            i = i + 1
                        if len(sig) > 0:
                            signatures.append(sig)
                            nsig = nsig + 1
            pduIdx = pduIdx + 1
        # // receive new messages
        slot = 0
        while slot < nslots:
            idx = ((slot) >> (3))
            bitVal = ((1) << ((((slot) & (7)))))
            prevBit = 0
            if idx < len(prevBitmap):
                prevBit = ((YGetByte(prevBitmap, idx)) & (bitVal))
            if (((YGetByte(newBitmap, idx)) & (bitVal))) != 0:
                if prevBit == 0:
                    sms = self.fetchPdu(slot)
                    newArr.append(sms)
                    if sms.get_concatCount() == 0:
                        newMsg.append(sms)
                    else:
                        sig = sms.get_concatSignature()
                        i = 0
                        while (i < nsig) and (len(sig) > 0):
                            if signatures[i] == sig:
                                sig = ""
                            i = i + 1
                        if len(sig) > 0:
                            signatures.append(sig)
                            nsig = nsig + 1
            slot = slot + 1
        
        self._pdus = newArr
        # // append complete concatenated messages
        i = 0
        while i < nsig:
            sig = signatures[i]
            cnt = 0
            pduIdx = 0
            while pduIdx < len(self._pdus):
                sms = self._pdus[pduIdx]
                if sms.get_concatCount() > 0:
                    if sms.get_concatSignature() == sig:
                        if cnt == 0:
                            cnt = sms.get_concatCount()
                            del newAgg[:]
                        newAgg.append(sms)
                pduIdx = pduIdx + 1
            if (cnt > 0) and (len(newAgg) == cnt):
                sms = YSms(self)
                sms.set_parts(newAgg)
                newMsg.append(sms)
            i = i + 1
        
        self._messages = newMsg
        
        return YAPI.SUCCESS

    def get_pdus(self):
        # // may throw an exception
        self.checkNewMessages()
        return self._pdus

    def clearPduCounters(self):
        """
        Clear the SMS units counters.

        @return YAPI.SUCCESS when the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        # retcode
        # // may throw an exception
        retcode = self.set_pduReceived(0)
        if retcode != YAPI.SUCCESS:
            return retcode
        retcode = self.set_pduSent(0)
        return retcode

    def sendTextMessage(self, recipient, message):
        """
        Sends a regular text SMS, with standard parameters. This function can send messages
        of more than 160 characters, using SMS concatenation. ISO-latin accented characters
        are supported. For sending messages with special unicode characters such as asian
        characters and emoticons, use newMessage to create a new message and define
        the content of using methods addText and addUnicodeData.

        @param recipient : a text string with the recipient phone number, either as a
                national number, or in international format starting with a plus sign
        @param message : the text to be sent in the message

        @return YAPI.SUCCESS when the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        # sms
        # // may throw an exception
        sms = YSms(self)
        sms.set_recipient(recipient)
        sms.addText(message)
        return sms.send()

    def sendFlashMessage(self, recipient, message):
        """
        Sends a Flash SMS (class 0 message). Flash messages are displayed on the handset
        immediately and are usually not saved on the SIM card. This function can send messages
        of more than 160 characters, using SMS concatenation. ISO-latin accented characters
        are supported. For sending messages with special unicode characters such as asian
        characters and emoticons, use newMessage to create a new message and define
        the content of using methods addText et addUnicodeData.

        @param recipient : a text string with the recipient phone number, either as a
                national number, or in international format starting with a plus sign
        @param message : the text to be sent in the message

        @return YAPI.SUCCESS when the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        # sms
        # // may throw an exception
        sms = YSms(self)
        sms.set_recipient(recipient)
        sms.set_msgClass(0)
        sms.addText(message)
        return sms.send()

    def newMessage(self, recipient):
        """
        Creates a new empty SMS message, to be configured and sent later on.

        @param recipient : a text string with the recipient phone number, either as a
                national number, or in international format starting with a plus sign

        @return YAPI.SUCCESS when the call succeeds.

        On failure, throws an exception or returns a negative error code.
        """
        # sms
        sms = YSms(self)
        sms.set_recipient(recipient)
        return sms

    def get_messages(self):
        """
        Returns the list of messages received and not deleted. This function
        will automatically decode concatenated SMS.

        @return an YSms object list.

        On failure, throws an exception or returns an empty list.
        """
        # // may throw an exception
        self.checkNewMessages()
        
        return self._messages

    def nextMessageBox(self):
        """
        Continues the enumeration of MessageBox interfaces started using yFirstMessageBox().

        @return a pointer to a YMessageBox object, corresponding to
                a MessageBox interface currently online, or a None pointer
                if there are no more MessageBox interfaces to enumerate.
        """
        hwidRef = YRefParam()
        if YAPI.YISERR(self._nextFunction(hwidRef)):
            return None
        if hwidRef.value == "":
            return None
        return YMessageBox.FindMessageBox(hwidRef.value)

#--- (end of generated code: YMessageBox implementation)

#--- (generated code: MessageBox functions)

    @staticmethod
    def FirstMessageBox():
        """
        Starts the enumeration of MessageBox interfaces currently accessible.
        Use the method YMessageBox.nextMessageBox() to iterate on
        next MessageBox interfaces.

        @return a pointer to a YMessageBox object, corresponding to
                the first MessageBox interface currently online, or a None pointer
                if there are none.
        """
        devRef = YRefParam()
        neededsizeRef = YRefParam()
        serialRef = YRefParam()
        funcIdRef = YRefParam()
        funcNameRef = YRefParam()
        funcValRef = YRefParam()
        errmsgRef = YRefParam()
        size = YAPI.C_INTSIZE
        #noinspection PyTypeChecker,PyCallingNonCallable
        p = (ctypes.c_int * 1)()
        err = YAPI.apiGetFunctionsByClass("MessageBox", 0, p, size, neededsizeRef, errmsgRef)

        if YAPI.YISERR(err) or not neededsizeRef.value:
            return None

        if YAPI.YISERR(
                YAPI.yapiGetFunctionInfo(p[0], devRef, serialRef, funcIdRef, funcNameRef, funcValRef, errmsgRef)):
            return None

        return YMessageBox.FindMessageBox(serialRef.value + "." + funcIdRef.value)

#--- (end of generated code: MessageBox functions)
